/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.plugin;

import java.net.URL;
import java.util.List;

import org.tinygroup.appconfig.AppConfig;
import org.tinygroup.context.Context;
import org.tinygroup.fileresolver.FileProcessor;
import org.tinygroup.plugin.config.PluginInfo;
import org.tinygroup.plugin.loader.PluginLoader;
import org.tinygroup.xmlparser.node.XmlNode;

public interface PluginManager extends SinglePluginManager, AllPluginManager,
		LevelPluginManager, AppConfig {
	String PLUGIN_XSTREAM = "plugin";
	int STATUS_UNINITIALIZED = 0;// 未初始化的
	int STATUS_INITING = 1;// 正在初始化
	int STATUS_INITED = 2;// 已初始化
	int STATUS_ASSEMBLEING = 3;// 正在装配
	int STATUS_ASSEMBLEED = 4;// 已装配
	int STATUS_STARTING = 5;// 正在开始中
	int STATUS_READY = 6;// 就绪
	int STATUS_PAUSING = 7;// 暂停中
	int STATUS_PAUSED = 8;// 已暂停
	int STATUS_STOPING = 9;// 正在停止中
	int STATUS_DISASSEMBLEING = 10;// 正在拆装
	int STATUS_DESTORYING = 11;// 正在执行销毁
	String PLUGIN_BASE_DIR = "basicDir";
	String WEBROOT = "TINY_WEBROOT";

	void setPluginLoader(PluginLoader pluginLoader);

	void add(PluginInfo pluginInfo);

	void add(PluginInfo[] pluginInfos);

	void add(List<PluginInfo> pluginInfolist);

	int status(PluginInfo pluginInfo);

	String getStatusDescription(int status);

	PluginInfo getPluginInfo(String id, String version);

	PluginInfo getPluginInfo(String id);

	List<PluginInfo> getAllPluginInfo();

	void remove(PluginInfo pluginInfo);

	void remove(List<PluginInfo> pluginInfolist);

	void removeAll();

	<T> T getService(PluginInfo pluginInfo, Class<T> clazz);

	<T> T getService(PluginInfo pluginInfo, Class<T> clazz, String version);

	<T> T getService(PluginInfo pluginInfo, String id);

	<T> T getService(PluginInfo pluginInfo, String id, String version);

	int size();

	/**
	 * 从指定路径加载插件
	 * 
	 * @param path
	 *            插件所在路径，可为目录或者单个jar文件
	 */
	void load(String path);

	/**
	 * 插件上下文传递接口
	 * 
	 * @return
	 */
	Context getContext();

	XmlNode getConifg(PluginInfo pluginInfo);

	/**
	 * 插件所需资源查找FileProcessor 主要用于当添加一个BU模块(或其他插件自定义模块)时，需要对BU的哪些插件相关资源进行扫描加载
	 * 
	 * @return
	 * List<FileProcessor> getPluginResourceProcessors();
	 */
	/**
	 * 插件所需资源移除FileProcessor 主要用于当移除一个BU模块(或其他插件自定义模块)时，需要移除BU的哪些已被加载到插件中相关资源
	 * 
	 * @return
	 * List<FileProcessor> getPluginRemoveResourceProcessors();
	 */
	
	/**
	 * 系统所需插件资源查找FileProcessor 主要用于当添加一个插件时，需要对插件的哪些相关资源进行扫描加载
	 * 
	 * @return
	 */
	List<FileProcessor> getPublicProcessors();

	/**
	 * 系统所需插件资源移除FileProcessor 主要用于当移除一个插件时，需要移除插件的哪些相关资源
	 * 
	 * @return
	 */
	List<FileProcessor> getPublicRemoveProcessors();

	// 添加系统资源扫描器
	void addPublicProcessor(FileProcessor processors);

	// 添加系统资源扫描器
	void addPublicRemoveProcessor(FileProcessor processors);

	void resolveResource(ClassLoader loader, URL urls1[]);

	void deResolveResource(ClassLoader loader, URL urls[]);
}
