/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
/*
 * 系统名称: JRES 应用快速开发企业套件
 * 模块名称: JRES内核
 * 文件名称: InitPluginStatus.java
 * 软件版权: 恒生电子股份有限公司
 * 修改记录:
 * 修改日期            修改人员                     修改说明 <br>
 * ========    =======  ============================================
 * 
 * ========    =======  ============================================
 */ 

package org.tinygroup.plugin.impl;

import static org.tinygroup.logger.LogLevel.INFO;

import java.util.List;

import org.tinygroup.logger.LogLevel;
import org.tinygroup.logger.Logger;
import org.tinygroup.logger.LoggerFactory;
import org.tinygroup.plugin.PluginManager;
import org.tinygroup.plugin.PluginStatus;
import org.tinygroup.plugin.config.PluginDependency;
import org.tinygroup.plugin.config.PluginInfo;
import org.tinygroup.plugin.exception.PluginException;

/**
 * 功能说明: 
 * <p> 系统版本: v1.0<br>
 * 开发人员: renhui <br>
 * 开发时间: 2012-9-19 <br>
 * 功能描述: 写明作用，调用方式，使用场景，以及特殊情况<br>
 */
public class InitPluginStatus implements PluginStatus
{
	private Logger logger = LoggerFactory.getLogger(InitPluginStatus.class);
	
	private PluginManager manager;
	private String status="init";
	
	/* (non-Javadoc)
	 * @see org.tinygroup.plugin.PluginStatus#actionStatus(org.tinygroup.plugin.PluginManager)
	 */
	public void actionStatus(PluginManager manager)
	{
		this.manager=manager;
		logger.logMessage(LogLevel.INFO, "开始初始化所有插件");
		List<PluginInfo> pluginInfoList=manager.getAllPluginInfo();
		for (PluginInfo pluginInfo : pluginInfoList) {
			try {
				init(pluginInfo);
			} catch (PluginException e) {
				processException(e);
			}
		}
		logger.logMessage(LogLevel.INFO, "所有插件已初始化");
	}

	/**
	 * method comments here
	 * @param e
	 */
	private void processException(PluginException e)
	{
		// TODO Auto-generated method stub
		
	}

	/**
	 * method comments here
	 * @param pluginInfo
	 */
	private void init(PluginInfo pluginInfo)
	{
//		logger.logMessage(LogLevel.INFO,
//				"开始初始化插件[id:{0}, version:{1}, type:{2}]", pluginInfo.getId(),
//				pluginInfo.getVersion(), pluginInfo.getType());
//
//		PluginStatus status = manager.getPluginStatus();
//		if (status.getStatus() != this.status) {
//			logger.logMessage(INFO, "插件[id:{0}, version:{1}]状态是[{2}],退出初始化",
//					pluginInfo.getId(), pluginInfo.getVersion(),
//					status.getStatus());
//			return;
//		}
//		initDependPlugins(pluginInfo);
//		// 初始化插件实类
//		try {
//			pluginMap.get(pluginInfo).init();
//
//		} catch (Exception e) {
//			setStatus(pluginInfo, status);
//			logger.logMessage(LogLevel.ERROR,
//					"插件[id:{0}, version:{1}, type:{2}]初始化时出错，恢复状态为[{3}]",
//					pluginInfo.getId(), pluginInfo.getVersion(),
//					pluginInfo.getType(), getStatusDescription(status));
//		}
//		logger.log(INFO, "plugin.init", pluginInfo.getId(),
//				pluginInfo.getVersion());
		
	}

	/* (non-Javadoc)
	 * @see org.tinygroup.plugin.PluginStatus#getStatus()
	 */
	public String getStatus()
	{
		return status;
	}
	
	private void initDependPlugins(PluginInfo pluginInfo){
		if (pluginInfo.getPluginDependencies() != null) {
			logger.logMessage(INFO, "开始初始化插件[id:{0}, version:{1}]依赖的项",
					pluginInfo.getId(), pluginInfo.getVersion());
			for (PluginDependency pluginDependency : pluginInfo
					.getPluginDependencies()) {
				PluginInfo dependPlugin = manager.getPluginInfo(
						pluginDependency.getPluginId(),
						pluginDependency.getPluginVersion());

				logger.logMessage(INFO,
						"开始初始化依赖的项[plugin-id:{0}, plugin-version:{1}]",
						dependPlugin.getId(), dependPlugin.getVersion());
				init(dependPlugin);
				logger.logMessage(INFO,
							"初始化依赖的项[plugin-id:{0}, plugin-version:{1}]完成",
							dependPlugin.getId(), dependPlugin.getVersion());
				
			}
			logger.logMessage(INFO, "初始化插件[id:{0}, version:{1}]依赖的项完成",
					pluginInfo.getId(), pluginInfo.getVersion());
		}
	}


}
